package net.drewke.tdme.engine;

/**
 * Frame buffer class
 * @author Andreas Drewke
 * @version $Id: 30105c0c654a74d550bd9bcbe3b623d712d3cd0f $
 */
public class FrameBuffer {

	private int width;
	private int height;
	private int frameBufferGlId;
	private int depthTextureGlId;

	/**
	 * Public constructor
	 * @param width
	 * @param height
	 */
	protected FrameBuffer(int width, int height) {
		this.width = width;
		this.height = height;
		frameBufferGlId = -1;
		depthTextureGlId = -1;
	}

	/**
	 * @return width
	 */
	protected int getWidth() {
		return width;
	}

	/**
	 * @return height
	 */
	protected int getHeight() {
		return height;
	}

	/**
	 * Init the frame buffer
	 * 	currently only depth buffer rendering is enabled, can be extended to color buffer easily
	 * @param gl
	 */
	protected void init() {
		depthTextureGlId = Engine.renderer.createDepthTexture(width, height);
		frameBufferGlId = Engine.renderer.createDepthFramebufferObject(depthTextureGlId);
	}

	/**
	 * Resize the frame buffer
	 * @param gl
	 */
	protected void reshape(int width, int height) {
		Engine.renderer.resizeDepthTexture(depthTextureGlId, width, height);
		this.width = width;
		this.height = height;
	}

	/**
	 * Disposes this frame buffer
	 * @param gl
	 */
	protected void dispose() {
		Engine.renderer.disposeTexture(depthTextureGlId);
		Engine.renderer.disposeFrameBufferObject(frameBufferGlId);
	}

	/**
	 * Enables this frame buffer to be rendered
	 * @param gl
	 */
	protected void enableFrameBuffer() {
		Engine.renderer.bindFrameBuffer(frameBufferGlId);
	}

	/**
	 * Disables any active frame buffer
	 * @param gl
	 */
	protected static void disableFrameBuffer() {
		Engine.renderer.bindFrameBuffer(Engine.renderer.ID_NONE);
	}

	/**
	 * Bind depth texture
	 * @param gl
	 */
	protected void bindDepthTexture() {
		Engine.renderer.bindTexture(depthTextureGlId);
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "FrameBuffer [width=" + width + ", height=" + height
				+ ", frameBufferGlId=" + frameBufferGlId
				+ ", depthTextureGlId=" + depthTextureGlId + "]";
	}

}
