package net.drewke.tdme.engine;

import net.drewke.tdme.model.FacesEntity;
import net.drewke.tdme.model.Material;

public class Object3DGroupDrawArrayRenderer implements Object3DGroupRenderer {

	private Object3DGroup object3DGroup;

	/**
	 * Constructor
	 * @param object 3D group
	 */
	protected Object3DGroupDrawArrayRenderer(Object3DGroup object3DGroup) {
		this.object3DGroup = object3DGroup;
	}

	/**
	 * Renders the object 3d group
	 * @param gl
	 */
	public void render() {
		// render faces entities
		FacesEntity[] facesEntities = object3DGroup.group.getFacesEntities();
		int faceIdx = 0;
		int faceIdxTextureCoordinates = 0;
		for (int i = 0; i < facesEntities.length; i++) {
			FacesEntity facesEntity = facesEntities[i];

			// material
			Material material = facesEntity.getMaterial();

			// set up material
			Object3DGroup.setupMaterial(object3DGroup, i);

			// optional texture coordinates
			if (object3DGroup.mesh.fbTextureCoordinates != null &&
				material != null &&
				material.hasDiffuseTexture() == true) {
				Engine.renderer.enableClientState(Engine.renderer.CLIENTSTATE_TEXTURECOORD_ARRAY);
				object3DGroup.mesh.fbTextureCoordinates.position(faceIdxTextureCoordinates * 3 * 2);
				Engine.renderer.setTextureCoordinatePointer(object3DGroup.mesh.fbTextureCoordinates);
			}
	
			// draw buffers
			//	vertices
			object3DGroup.mesh.fbVertices.position(faceIdx * 3 * 3);
			Engine.renderer.setVerticesPointer(object3DGroup.mesh.fbVertices);

			//	normals
			object3DGroup.mesh.fbNormals.position(faceIdx * 3 * 3);
			Engine.renderer.setNormalsPointer(object3DGroup.mesh.fbNormals);

			// draw from buffers
			Engine.renderer.drawTrianglesFromBufferPointers(facesEntity.getFaces().length);

			// keep track of rendered faces
			faceIdx+= facesEntity.getFaces().length;

			// optional texture coordinates
			if (object3DGroup.mesh.fbTextureCoordinates != null) {
				Engine.renderer.disableClientState(Engine.renderer.CLIENTSTATE_TEXTURECOORD_ARRAY);

				// keep track of rendered faces with texture coordinates
				faceIdxTextureCoordinates+= facesEntity.getFaces().length;
			}

			// clear material
			Object3DGroup.clearMaterial();
		}
	}

	/**
	 * Disposes the object 3d group
	 * @param gl
	 */
	public void dispose() {
	}

}
