package net.drewke.tdme.engine;

import net.drewke.tdme.model.FacesEntity;
import net.drewke.tdme.model.Material;

public class Object3DGroupVBORenderer implements Object3DGroupRenderer {

	private Object3DGroup object3DGroup;
	private int[] vboGlIds;

	/**
	 * Constructor
	 * @param object 3D group
	 */
	protected Object3DGroupVBORenderer(Object3DGroup object3DGroup) {
		this.object3DGroup = object3DGroup;
		this.vboGlIds = null;
	}

	/**
	 * Renders the object 3d group
	 * @param gl
	 */
	public void render() {
		// initialize if not yet done
		if (vboGlIds == null) {
			vboGlIds = Engine.getInstance().getVBOManager().addVBO(object3DGroup.id, object3DGroup.mesh.fbTextureCoordinates != null?3:2);
		}

		// check if to upload new mesh
		if (object3DGroup.mesh.hasRecreatedBuffers() == true) {
			// upload vertices
			Engine.renderer.uploadBufferObject(
				vboGlIds[0],
				object3DGroup.mesh.fbVertices.capacity() * Float.SIZE / Byte.SIZE,
				object3DGroup.mesh.fbVertices
			);

			// upload normals
			Engine.renderer.uploadBufferObject(
				vboGlIds[1],
				object3DGroup.mesh.fbNormals.capacity() * Float.SIZE / Byte.SIZE,
				object3DGroup.mesh.fbNormals
			);

			// upload texture coordinates
			if (object3DGroup.mesh.fbTextureCoordinates != null) {
				Engine.renderer.uploadBufferObject(
					vboGlIds[2],
					object3DGroup.mesh.fbTextureCoordinates.capacity() * Float.SIZE / Byte.SIZE,
					object3DGroup.mesh.fbTextureCoordinates
				);
			}
		}

		// render faces entities
		FacesEntity[] facesEntities = object3DGroup.group.getFacesEntities();
		int faceIdx = 0;
		int faceIdxTextureCoordinates = 0;
		for (int i = 0; i < facesEntities.length; i++) {
			FacesEntity facesEntity = facesEntities[i];

			// material
			Material material = facesEntity.getMaterial();

			// set up material
			Object3DGroup.setupMaterial(object3DGroup, i);

			// draw buffers
			if (vboGlIds.length > 2 &&
				material != null &&
				material.hasDiffuseTexture() == true) {
				Engine.renderer.enableClientState(Engine.renderer.CLIENTSTATE_TEXTURECOORD_ARRAY);
				Engine.renderer.bindTextureCoordinatesBufferObject(vboGlIds[2], faceIdxTextureCoordinates);
			}
			Engine.renderer.bindVerticesBufferObject(vboGlIds[0], faceIdx);
			Engine.renderer.bindNormalsBufferObject(vboGlIds[1], faceIdx);
			Engine.renderer.drawTrianglesFromBufferObjects(facesEntity.getFaces().length);

			// keep track of rendered faces
			faceIdx+= facesEntity.getFaces().length;

			// disable texture client state
			if (vboGlIds.length > 2 &&
				material != null &&
				material.hasDiffuseTexture() == true) {
				Engine.renderer.enableClientState(Engine.renderer.CLIENTSTATE_TEXTURECOORD_ARRAY);
				faceIdxTextureCoordinates+= facesEntity.getFaces().length;
			}

			// clear material
			Object3DGroup.clearMaterial();
		}

		// unbind buffers
		Engine.renderer.unbindBufferObjects();
	}

	/**
	 * Disposes the object 3d group
	 * @param gl
	 */
	public void dispose() {
		// 
		if (vboGlIds != null) {
			Engine.getInstance().getVBOManager().removeVBO(object3DGroup.id);
			vboGlIds = null;
		}
	}

}
