package net.drewke.tdme.engine;

import javax.media.opengl.GL2;
import javax.media.opengl.glu.GLU;
import javax.media.opengl.glu.GLUquadric;

import net.drewke.tdme.math.Vector3D;
import net.drewke.tdme.primitives.BoundingBox;
import net.drewke.tdme.primitives.BoundingVolume;
import net.drewke.tdme.primitives.Capsule;
import net.drewke.tdme.primitives.OrientedBoundingBox;
import net.drewke.tdme.primitives.Sphere;

/**
 * Helper to render primitives
 * @author Andreas Drewke
 * @version $Id: 7446a46d55663d04bb933c12b7543df70e69f510 $
 */
public class Primitives {

	/**
	 * Renders a bounding box
	 * @param gl
	 * @param bb
	 */
	public static void doRenderBoundingBox(GL2 gl, BoundingBox boundingBox) {
		// 
		float min[] = boundingBox.getMin().getArray();
		float max[] = boundingBox.getMax().getArray();
		float minX = min[0], minY = min[1], minZ = min[2];
		float maxX = max[0], maxY = max[1], maxZ = max[2];

		// draw bounding box
		gl.glBegin(GL2.GL_LINES);
		//	front rectangle on xy plane
		//		*-----*
		//		|     |
		//		-------
		gl.glVertex3f(minX, minY, maxZ);
		gl.glVertex3f(maxX, minY, maxZ);
		//		-------
		//		|     |
		//		*-----*
		gl.glVertex3f(minX, maxY, maxZ);
		gl.glVertex3f(maxX, maxY, maxZ);
		//		*------
		//		|     |
		//		*------
		gl.glVertex3f(minX, minY, maxZ);
		gl.glVertex3f(minX, maxY, maxZ);
		//		------*
		//		|     |
		//		------*
		gl.glVertex3f(maxX, minY, maxZ);
		gl.glVertex3f(maxX, maxY, maxZ);
		//	back rectangle on xy plane
		//		*-----*
		//		|     |
		//		-------
		gl.glVertex3f(minX, minY, minZ);
		gl.glVertex3f(maxX, minY, minZ);
		//		-------
		//		|     |
		//		*-----*
		gl.glVertex3f(minX, maxY, minZ);
		gl.glVertex3f(maxX, maxY, minZ);
		//		*------
		//		|     |
		//		*------
		gl.glVertex3f(minX, minY, minZ);
		gl.glVertex3f(minX, maxY, minZ);
		//		------*
		//		|     |
		//		------*
		gl.glVertex3f(maxX, minY, minZ);
		gl.glVertex3f(maxX, maxY, minZ);
		
		// connections left rectangle yz plane
		gl.glVertex3f(minX, minY, maxZ);
		gl.glVertex3f(minX, minY, minZ);
		gl.glVertex3f(minX, maxY, maxZ);
		gl.glVertex3f(minX, maxY, minZ);

		// connections right rectangle yz plane
		gl.glVertex3f(maxX, minY, maxZ);
		gl.glVertex3f(maxX, minY, minZ);
		gl.glVertex3f(maxX, maxY, maxZ);
		gl.glVertex3f(maxX, maxY, minZ);

		gl.glEnd();
	}

	/**
	 * Renders a oriented bounding box
	 * @param gl
	 * @param obb
	 */
	public static void doRenderOrientedBoundingBox(GL2 gl, OrientedBoundingBox obb) {
		Vector3D[] vertices = obb.computeVertices();

		// draw bounding box
		gl.glBegin(GL2.GL_LINES);
		gl.glVertex3f(vertices[0].getX(), vertices[0].getY(), vertices[0].getZ());
		gl.glVertex3f(vertices[1].getX(), vertices[1].getY(), vertices[1].getZ());
		gl.glVertex3f(vertices[1].getX(), vertices[1].getY(), vertices[1].getZ());
		gl.glVertex3f(vertices[2].getX(), vertices[2].getY(), vertices[2].getZ());
		gl.glVertex3f(vertices[2].getX(), vertices[2].getY(), vertices[2].getZ());
		gl.glVertex3f(vertices[3].getX(), vertices[3].getY(), vertices[3].getZ());
		gl.glVertex3f(vertices[3].getX(), vertices[3].getY(), vertices[3].getZ());
		gl.glVertex3f(vertices[0].getX(), vertices[0].getY(), vertices[0].getZ());
		gl.glVertex3f(vertices[4].getX(), vertices[4].getY(), vertices[4].getZ());
		gl.glVertex3f(vertices[5].getX(), vertices[5].getY(), vertices[5].getZ());
		gl.glVertex3f(vertices[5].getX(), vertices[5].getY(), vertices[5].getZ());
		gl.glVertex3f(vertices[6].getX(), vertices[6].getY(), vertices[6].getZ());
		gl.glVertex3f(vertices[6].getX(), vertices[6].getY(), vertices[6].getZ());
		gl.glVertex3f(vertices[7].getX(), vertices[7].getY(), vertices[7].getZ());
		gl.glVertex3f(vertices[7].getX(), vertices[7].getY(), vertices[7].getZ());
		gl.glVertex3f(vertices[4].getX(), vertices[4].getY(), vertices[4].getZ());
		gl.glVertex3f(vertices[0].getX(), vertices[0].getY(), vertices[0].getZ());
		gl.glVertex3f(vertices[4].getX(), vertices[4].getY(), vertices[4].getZ());
		gl.glVertex3f(vertices[1].getX(), vertices[1].getY(), vertices[1].getZ());
		gl.glVertex3f(vertices[5].getX(), vertices[5].getY(), vertices[5].getZ());
		gl.glVertex3f(vertices[3].getX(), vertices[3].getY(), vertices[3].getZ());
		gl.glVertex3f(vertices[7].getX(), vertices[7].getY(), vertices[7].getZ());
		gl.glVertex3f(vertices[2].getX(), vertices[2].getY(), vertices[2].getZ());
		gl.glVertex3f(vertices[6].getX(), vertices[6].getY(), vertices[6].getZ());
		gl.glEnd();
	}

	/**
	 * Renders a sphere
	 * @param gl
	 * @param sphere
	 */
	public static void doRenderSphere(GL2 gl, Sphere sphere) {
		Vector3D center = sphere.getCenter();
		float radius = sphere.getRadius();
		gl.glPushMatrix();
		gl.glTranslatef(center.getX(), center.getY(), center.getZ());
		Engine.getGLUT().glutWireSphere(radius, 6, 6);
		gl.glPopMatrix();
	}

	/**
	 * Renders a capsule
	 * @param gl
	 * @param capsule
	 */
	public static void doRenderCapsule(GL2 gl, Capsule capsule) {
		GLU glu = Engine.getGLU();

		Vector3D a = capsule.getA();
		Vector3D b = capsule.getB();
		float radius = capsule.getRadius();

		gl.glColor4f(1f, 0f, 0f, 0.5f);

		// draw a sphere
		gl.glPushMatrix();
		gl.glTranslatef(a.getX(), a.getY(), a.getZ());
		Engine.getGLUT().glutSolidSphere(radius, 24, 24);
		gl.glPopMatrix();

		// draw b sphere
		gl.glPushMatrix();
		gl.glTranslatef(b.getX(), b.getY(), b.getZ());
		Engine.getGLUT().glutSolidSphere(radius, 24, 24);
		gl.glPopMatrix();

		// draw cylinder
		Vector3D z = new Vector3D(0f,0f,1f);
		Vector3D p = a.clone().sub(b);
		float pLength = p.computeLength();
		Vector3D t = Vector3D.computeCrossProduct(z, p);
		float angle = 0f;
		if (pLength > 0f) {
			angle = (float)(
				180d /
				Math.PI *
				Math.acos(
					Vector3D.computeDotProduct(z, p) / pLength
				)
			);
		}
		gl.glPushMatrix();
		gl.glTranslatef(b.getX(), b.getY(), b.getZ());
		gl.glRotatef(angle, t.getX(), t.getY(), t.getZ());
		GLUquadric gluQuadric = glu.gluNewQuadric();
		glu.gluQuadricOrientation(gluQuadric, GLU.GLU_OUTSIDE);
		glu.gluCylinder(gluQuadric, radius, radius, pLength, 24, 24);
		glu.gluDeleteQuadric(gluQuadric);
		gl.glPopMatrix();
	}

	/**
	 * Renders a bounding volume
	 * @param gl
	 * @param bounding volume
	 */
	public static void doRenderBoundingVolume(GL2 gl, BoundingVolume boundingVolume) {
		if (boundingVolume instanceof BoundingBox) {
			doRenderBoundingBox(gl, (BoundingBox)boundingVolume);
		} else
		if (boundingVolume instanceof OrientedBoundingBox) {
			doRenderOrientedBoundingBox(gl, (OrientedBoundingBox)boundingVolume);
		} else
		if (boundingVolume instanceof Sphere) {
			doRenderSphere(gl, (Sphere)boundingVolume);
		} else
		if (boundingVolume instanceof Capsule) {
			doRenderCapsule(gl, (Capsule)boundingVolume);
		} else {
			System.out.println("Warning: Primitives::doRenderBoundingVolume(): bounding volume not supported.");
		}
	}

}