package net.drewke.tdme.engine;

import net.drewke.tdme.math.Matrix3D;
import net.drewke.tdme.math.Vector3D;

/**
 * Rotation
 * @author Andreas Drewke
 * @version $Id: 7568c0bd9d7c9a1b723f2260867741948703b137 $
 */
public class Rotation {

	private float angle;
	private Vector3D axis;
	private Matrix3D matrix;

	/**
	 * Public default constructor
	 */
	public Rotation() {
		this.angle = 0f;
		this.axis = new Vector3D(0f, 0f, 0f);		
		this.matrix = new Matrix3D().identity();
	}

	/**
	 * Public constructor
	 * @param angle
	 * @param axis
	 */
	public Rotation(float angle, Vector3D axis) {
		this.angle = angle;
		this.axis = axis;
		this.matrix = new Matrix3D().identity();
	}

	/**
	 * @return angle
	 */
	public float getAngle() {
		return angle;
	}

	/**
	 * @param angle
	 */
	public void setAngle(float angle) {
		this.angle = angle;
	}

	/**
	 * @return axis
	 */
	public Vector3D getAxix() {
		return axis;
	}

	/**
	 * @return rotation matrix
	 */
	public Matrix3D getMatrix() {
		return matrix;
	}

	/**
	 * Sets up this rotation from another rotation
	 * @param rotation
	 */
	public void fromRotation(Rotation rotation) {
		angle = rotation.angle;
		axis.set(rotation.axis);
		matrix.set(rotation.matrix);
	}

	/**
	 * Computes rotation matrix
	 */
	public void update() {
		matrix.identity();
		matrix.rotate(angle, axis);		
	}

}