package net.drewke.tdme.engine;

import java.util.Hashtable;

import net.drewke.tdme.assets.textures.Texture;

/**
 * Texture manager
 * @author Andreas Drewke
 * @version $Id: dbdd5a6ae7109b295e0a83274ec87605ed841eb6 $
 */
public class TextureManager {

	/**
	 * Managed texture entity
	 * @author Andreas Drewke
	 */
	private class TextureManaged {

		String id;
		int glId;
		int referenceCounter;

		/**
		 * Protected constructor
		 * @param id
		 * @param glId
		 * @param referenceCounter
		 */
		private TextureManaged(String id, int glId) {
			this.id = id;
			this.glId = glId;
			this.referenceCounter = 0;
		}

		/**
		 * @return texture id
		 */
		private String getId() {
			return id;
		}

		/**
		 * @return texture open gl id
		 */
		private int getGlId() {
			return glId;
		}

		/**
		 * @return reference counter
		 */
		private int getReferenceCounter() {
			return referenceCounter;
		}

		/**
		 * decrement reference counter
		 * @return if reference counter = 0
		 */
		private boolean decrementReferenceCounter() {
			referenceCounter--;
			return referenceCounter == 0;
		}

		/**
		 * increment reference counter
		 */
		private void incrementReferenceCounter() {
			referenceCounter++;
		}

	}

	private Hashtable<String, TextureManaged> textures;

	/**
	 * Public constructor
	 */
	protected TextureManager() {
		textures = new Hashtable<String, TextureManaged>(); 
	}

	/**
	 * Adds a texture to manager / open gl stack
	 * @param gl
	 * @param texture
	 * @param gl texture id
	 */
	protected int addTexture(Texture texture) {
		// check if we already manage this texture
		TextureManaged textureManaged = textures.get(texture.getId());
		if (textureManaged != null) {
			//
			textureManaged.incrementReferenceCounter();

			// yep, return open gl id
			return textureManaged.getGlId();
		}

		// create texture
		int textureId = Engine.renderer.createTexture();

		// bind texture
		Engine.renderer.bindTexture(textureId);

		// upload texture
		Engine.renderer.uploadTexture(texture);

		// unbind texture
		Engine.renderer.bindTexture(Engine.renderer.ID_NONE);


		// create managed texture
		textureManaged = new TextureManaged(
			texture.getId(),
			textureId
		);
		textureManaged.incrementReferenceCounter();

		// add it to our textures
		textures.put(
			texture.getId(),
			textureManaged
		);

		// return open gl id
		return textureId;
	}

	/**
	 * Removes a texture from manager / open gl stack
	 * @param gl
	 * @param texture
	 */
	protected void removeTexture(String textureId) {
		TextureManaged textureManaged = textures.get(textureId);
		if (textureManaged != null) {
			if (textureManaged.decrementReferenceCounter()) {
				// delete texture
				Engine.renderer.disposeTexture(textureManaged.getGlId());
				// remove from our list
				textures.remove(textureId);
			}
			return;
		}
		System.out.println("Warning: texture not loaded by texture manager");
	}

}