package net.drewke.tdme.engine;

import java.util.Vector;

import net.drewke.tdme.math.Matrix3D;
import net.drewke.tdme.math.Vector3D;

/**
 * Transformations
 * @author Andreas Drewke
 * @version $Id: a1e7fce526fa26bac7dfe14ffba64ae8cefd11fa $
 */
public class Transformations {

	protected Vector3D translation;
	protected Matrix3D translationMatrix;

	protected Vector3D scale;
	protected Matrix3D scaleMatrix;

	protected Vector<Rotation> rotations;

	protected Matrix3D transformationsMatrix;

	/**
	 * Public constructor
	 */
	public Transformations() {
		transformationsMatrix = new Matrix3D().identity();
		translation = new Vector3D();
		translationMatrix = new Matrix3D();
		scale = new Vector3D(1f,1f,1f);
		scaleMatrix = new Matrix3D();
		rotations = new Vector<Rotation>();
	}

	/**
	 * @return object translation
	 */
	public Vector3D getTranslation() {
		return translation;
	}

	/** 
	 * @return object scale
	 */
	public Vector3D getScale() {
		return scale;
	}

	/**
	 * @return object rotations
	 */
	public Vector<Rotation> getRotations() {
		return rotations;
	}

	/**
	 * @return this transformations matrix
	 */
	public Matrix3D getMatrix() {
		return transformationsMatrix; 
	}

	/**
	 * Set up this transformations from given transformations
	 * @param transformations
	 */
	public void fromTransformations(Transformations transformations) {
		// translation
		translation.set(transformations.translation);

		// scale
		scale.set(transformations.scale);

		// rotations
		int rotationIdx = 0;
		for(Rotation rotation: transformations.rotations) {
			// do we have a rotation to reuse?
			Rotation _rotation = rotationIdx < rotations.size()?rotations.elementAt(rotationIdx):null;
			//	nope?
			if (_rotation == null) {
				// add it
				_rotation = new Rotation();
				rotations.add(_rotation);
			}
			// copy
			_rotation.fromRotation(rotation);
			//
			rotationIdx++;
		}

		// remove unused rotations
		while (rotationIdx < rotations.size()) rotations.remove(rotations.size() - 1);

		// copy matrices
		translationMatrix.set(transformations.translationMatrix);
		scaleMatrix.set(transformations.scaleMatrix);
		transformationsMatrix.set(transformations.transformationsMatrix);
	}

	/**
	 * Computes transformation matrix
	 */
	public void update() {
		// set up translation matrix
		translationMatrix.identity().translate(translation);

		// set up scale matrix
		scaleMatrix.identity().scale(scale);

		// apply transformation matrices
		transformationsMatrix.identity();
		transformationsMatrix.multiply(scaleMatrix);
		for(Rotation rotation: rotations) {
			rotation.update();
			transformationsMatrix.multiply(rotation.getMatrix());
		}
		transformationsMatrix.multiply(translationMatrix);
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "Transformations [translation=" + translation
				+ ", translationMatrix=" + translationMatrix + ", scale="
				+ scale + ", scaleMatrix=" + scaleMatrix + ", rotations="
				+ rotations + ", transformationsMatrix="
				+ transformationsMatrix + "]";
	}

}
