package net.drewke.tdme.engine;

import java.util.Hashtable;

/**
 * VBO manager
 * @author Andreas Drewke
 * @version $Id: f4c6fa473cae45c655817f37705706937b5819f1 $
 */
public class VBOManager {

	/**
	 * Managed VBO entity
	 * @author Andreas Drewke
	 */
	private class VBOManaged {

		String id;
		int[] vboGlIds;
		int referenceCounter;

		/**
		 * Protected constructor
		 * @param id
		 * @param vbo gl id
		 * @param referenceCounter
		 */
		private VBOManaged(String id, int[] vboGlIds) {
			this.id = id;
			this.vboGlIds = vboGlIds;
			this.referenceCounter = 0;
		}

		/**
		 * @return vbo id
		 */
		private String getId() {
			return id;
		}

		/**
		 * @return vbo gl ids
		 */
		private int[] getVBOGlIds() {
			return vboGlIds;
		}

		/**
		 * @return reference counter
		 */
		private int getReferenceCounter() {
			return referenceCounter;
		}

		/**
		 * decrement reference counter
		 * @return if reference counter = 0
		 */
		private boolean decrementReferenceCounter() {
			referenceCounter--;
			return referenceCounter == 0;
		}

		/**
		 * increment reference counter
		 */
		private void incrementReferenceCounter() {
			referenceCounter++;
		}

	}

	private Hashtable<String, VBOManaged> vbos;

	/**
	 * Public constructor
	 */
	protected VBOManager() {
		vbos = new Hashtable<String, VBOManaged>(); 
	}

	/**
	 * Adds a vbo to manager / open gl stack
	 * @param gl
	 * @param texture
	 * @param vbo id count
	 */
	protected int[] addVBO(String vboId, int ids) {
		// check if we already manage this vbo
		VBOManaged vboManaged = vbos.get(vboId);
		if (vboManaged != null) {
			//
			vboManaged.incrementReferenceCounter();

			// yep, return vbo id
			return vboManaged.getVBOGlIds();
		}

		// create vertex buffer objects
		int[] vboIds = Engine.renderer.createBufferObjects(ids);

		// create managed texture
		vboManaged = new VBOManaged(
			vboId,
			vboIds
		);
		vboManaged.incrementReferenceCounter();

		// add it to our textures
		vbos.put(
			vboManaged.getId(),
			vboManaged
		);

		// return open gl id
		return vboIds;
	}

	/**
	 * Removes a vbo from manager / open gl stack
	 * @param gl
	 * @param texture
	 */
	protected void removeVBO(String vboId) {
		VBOManaged vboManaged = vbos.get(vboId);
		if (vboManaged != null) {
			if (vboManaged.decrementReferenceCounter()) {
				// delete vbos from open gl
				int[] vboIds = vboManaged.getVBOGlIds();
				Engine.renderer.disposeBufferObjects(vboIds);
			}
			return;
		}
		System.out.println("Warning: vbo not managed by vbo manager");
	}

}