package net.drewke.tdme.math;

import java.util.Arrays;

/**
 * Separated axis test
 * 	ported from "game physics - a practical introduction/ben kenwright"
 * @author Andreas Drewke
 * @version $Id: 1c8fa2f5ad23f79da97f5b74e5fd053f48c3d5a7 $
 */
public class SeparatingAxisTheorem {

	/**
	 * Projects the vertices onto the plane and returns the minimum and maximum values
	 * 	ported from "game physics - a practical introduction/ben kenwright"
	 * @param obb
	 * @param axis
	 * @return float[] containing min and max
	 */
	private static float[] doCalculateInterval(Vector3D[] vertices, Vector3D axis) {
		float distance = Vector3D.computeDotProduct(vertices[0], axis);
		float min = distance;
		float max = distance;
		for (int i = 1; i < vertices.length; i++) {
			distance = Vector3D.computeDotProduct(vertices[i], axis);
			if (distance < min) min = distance;
			if (distance > max) max = distance;
		}

		// return min, max
		float[] result = {min, max};
		return result;
	}

	/**
	 * Determines penetration of given vertices for both objects on a given axis
	 * 	based on an algorithm from "game physics - a practical introduction/ben kenwright"
	 * @param vertices 1
	 * @param vertices 2
	 * @param axis test
	 * @param axis penetration
	 * @return penetration or negative / -1 if none
	 */
	public static float doSpanIntersect(Vector3D[] vertices1, Vector3D[] vertices2, Vector3D axisTest, Vector3D axisPenetration) {
		Vector3D axis = axisTest.clone().normalize();
		axisPenetration.set(axis);

		// min, max for vertices 1 on axis
		float[] minMax1 = doCalculateInterval(vertices1, axis);

		// min, max for vertices 2 on axis
		float[] minMax2 = doCalculateInterval(vertices2, axis);

		// determine penetration
		float min1 = minMax1[0]; 
		float max1 = minMax1[1];
		float min2 = minMax2[0]; 
		float max2 = minMax2[1];
		float len1 = max1 - min1;
		float len2 = max2 - min2;
		float penetration;

		// check if we have no penetration
		if (max1 < min2 || min1 > max2) {
			return -1f;
		} else
		// determine if object1 is in object2
		if (min1 > min2 && max1 < max2) {
			penetration = len1 + Math.min(Math.abs(min1 - min2), Math.abs(max1 - max2));
		} else
		// determine if object2 is in object1
		if (min2 > min1 && max2 < max1) {
			penetration = len2 + Math.min(Math.abs(min1 - min2), Math.abs(max1 - max2));
		} else {
			// one object just touches the other
			float min = Math.min(min1, min2);
			float max = Math.max(max1, max2);
			float len = max - min;
			penetration = (len1 + len2) - len;
		}

		//
		if (min2 < min1) {
			axisPenetration.scale(-1f);
		}

		return penetration;
	}

}
