package net.drewke.tdme.model;

import java.util.Vector;

import net.drewke.tdme.math.Matrix3D;

/**
 * AnimationSetup
 * @author andreas.drewke
 * @version $Id: 327ff8ab0df6f965f1acb61b0266a60a41aebd59 $
 */
public class Animation {

	private float fps;
	private int frames;
	private Matrix3D[] transformationsMatrices;

	/**
	 * Public constructor
	 * @param frames per second
	 * @param frames
	 */
	public Animation(float fps, int frames) {
		this.fps = fps;
		this.frames = frames;
		transformationsMatrices = new Matrix3D[this.frames];
		for(int i = 0; i < frames; i++) {
			transformationsMatrices[i] = new Matrix3D().identity();
		}
	}

	/**
	 * @return frames per second
	 */
	public float getFPS() {
		return fps;
	}

	/**
	 * @return number of frames
	 */
	public int getFrames() {
		return frames;
	}

	/**
	 * Returns transformation matrices
	 * @return transformation matrices
	 */
	public Matrix3D[] getTransformationsMatrices() {
		return transformationsMatrices;
	}

	/**
	 * @return string representation
	 */
	public String toString() {
		return "AnimationSetup [frames=" + frames + ", transformationsMatrices=" + transformationsMatrices + "]";
	}

}