package net.drewke.tdme.model;

import java.util.Arrays;
import java.util.Vector;

/**
 * Group faces entity
 * 	A group can have multiple entities containing faces and a applied material
 * @author Andreas Drewke
 * @version $Id: 291faf750c322df9b93e225bb38f7df6d50fba05 $
 */
public class FacesEntity {

	private String id;
	private Group group;
	private Material material;
	private Face[] faces;

	/**
	 * Public constructor
	 * @param id 
	 * @param group
	 */
	public FacesEntity(Group group, String id) {
		this.id = id;
		this.group = group;
		this.material = null;
		this.faces = new Face[0];
	}

	/**
	 * @return faces entity id
	 */
	public String getId() {
		return id;
	}

	/**
	 * Set up the entity's material
	 * @param material
	 */
	public void setMaterial(Material material) {
		this.material = material;
	}

	/**
	 * @return entity's material
	 */
	public Material getMaterial() {
		return material;
	}

	/**
	 * @return entity's faces
	 */
	public Face[] getFaces() {
		return faces;
	}

	/**
	 * Set up entity's faces
	 * @param faces
	 */
	public void setupFaces(Vector<Face> faces) {
		this.faces = faces.toArray(new Face[faces.size()]);
	}

	/**
	 * @return string representation
	 */
	public String toString() {
		return "FacesEntity [id=" + id + ", group=" + group.getName()
				+ ", material=" + material + ", faces="
				+ Arrays.toString(faces) + "]";
	}

}