package net.drewke.tdme.model;

import java.util.Hashtable;

import net.drewke.tdme.math.Matrix3D;

/**
 * Represents a 3d model
 * @author andreas.drewke
 * @version $Id: 317d7820a8fea222ced2d0dffd96312a7fe02eec $
 */
public class Model {

	public final static String ANIMATIONSETUP_DEFAULT = "tdme.default";
	public final static float FPS_DEFAULT = 30f;

	private String id;
	private String name;
	private Hashtable<String, Material> materials;
	private Hashtable<String, Group> groups;
	private Hashtable<String, Group> subGroups;

	private float fps;
	private Hashtable<String, AnimationSetup> animationSetups;

	private Matrix3D importTransformationsMatrix;

	/**
	 * Public constructor
	 * @param id
	 * @param name
	 */
	public Model(String id, String name) {
		this.id = id;
		this.name = name;
		materials = new Hashtable<String, Material>();
		groups = new Hashtable<String, Group>();
		subGroups = new Hashtable<String, Group>();
		fps = FPS_DEFAULT;
		animationSetups = new Hashtable<String, AnimationSetup>();
		importTransformationsMatrix = new Matrix3D().identity();
	}

	/**
	 * @return model id
	 */
	public String getId() {
		return id;
	}

	/**
	 * @return model name
	 */
	public String getName() {
		return name;
	}

	/**
	 * Returns all object materials
	 * @return materials
	 */
	public Hashtable<String, Material> getMaterials() {
		return materials;
	}

	/**
	 * Returns all object's groups
	 * @return all groups
	 */
	public Hashtable<String, Group> getGroups() {
		return groups;
	}

	/**
	 * Returns a group by given name or null
	 * @param id
	 * @return
	 */
	public Group getGroupById(String id) {
		return groups.get(id);
	}

	/**
	 * Returns object's sub groups
	 * @return sub groups
	 */
	public Hashtable<String, Group> getSubGroups() {
		return subGroups;
	}

	/**
	 * Returns a sub group by given name or null
	 * @param id
	 * @return
	 */
	public Group getSubGroupById(String id) {
		return subGroups.get(id);
	}

	/**
	 * @return frames per seconds
	 */
	public float getFPS() {
		return fps;
	}

	/**
	 * Set model animation frames per seconds
	 * @param fps
	 */
	public void setFPS(float fps) {
		this.fps = fps;
	}

	/**
	 * Adds an animation setup
	 * @param id
	 * @param start frame
	 * @param end frame
	 * @param loop
	 * @return success
	 */
	public boolean addAnimationSetup(String id, int startFrame, int endFrame, boolean loop) { 
		animationSetups.put(id, new AnimationSetup(id, startFrame, endFrame, loop));
		return true;
	}

	/**
	 * @return animation setup for given id or null
	 */
	public AnimationSetup getAnimationSetup(String id) {
		return animationSetups.get(id);
	}

	/**
	 * @return if model has animations
	 */
	public boolean hasAnimations() {
		return animationSetups.size() > 0;
	}

	/**
	 * @return import transformations matrix like converting Z-UP to Y-UP
	 */
	public Matrix3D getImportTransformationsMatrix() {
		return importTransformationsMatrix;
	}

	/**
	 * @return string representation
	 */
	public String toString() {
		return "Model [name=" + name +
				", subGroups=" + subGroups + "]";
	}

}