package net.drewke.tdme.model;

import java.util.Arrays;
import java.util.Vector;

import net.drewke.tdme.math.Matrix3D;
import net.drewke.tdme.math.Vector3D;

/**
 * Skinning definition for groups
 * @author andreas.drewke
 * @version $Id: 61285cdeb87e14593582c97f6ec981e2bba54dcd $
 */
public class Skinning {

	// weights
	private float[] weights;

	// joints
	private Joint[] joints;

	// for each vertex we have a set of joints with a corresponding weight
	private JointWeight[][] verticesJointsWeights;

	/**
	 * Public constructor
	 */
	public Skinning() {
		weights = new float[0];
		joints = new Joint[0];
		verticesJointsWeights = new JointWeight[0][0];
	}

	/**
	 * @return weights
	 */
	public float[] getWeights() {
		return weights;
	}

	/**
	 * Set up weights
	 * @param weights
	 */
	public void setupWeights(Vector<Float> weights) {
		this.weights = new float[weights.size()];
		for(int i = 0; i < this.weights.length; i++) {
			this.weights[i] = weights.get(i);
		}
	}

	/**
	 * @return all joints
	 */
	public Joint[] getJoints() {
		return joints;
	}

	/**
	 * Set up joints
	 * @param joints
	 */
	public void setupJoints(Vector<Joint> joints) {
		this.joints = joints.toArray(new Joint[joints.size()]);
	}

	/**
	 * @return all vertex joints
	 */
	public JointWeight[][] getVerticesJointsWeights() {
		return verticesJointsWeights;
	}

	/**
	 * Sets up vertices joints weights 
	 * @param verticesJointsWeights
	 */
	public void setupVerticesJointsWeights(Vector<Vector<JointWeight>> verticesJointsWeights) {
		this.verticesJointsWeights = new JointWeight[verticesJointsWeights.size()][];
		for (int i = 0; i < verticesJointsWeights.size(); i++) {
			this.verticesJointsWeights[i] = new JointWeight[verticesJointsWeights.get(i).size()];
			for (int j = 0; j < verticesJointsWeights.get(i).size(); j++) {
				this.verticesJointsWeights[i][j] = verticesJointsWeights.get(i).get(j);
			}
		}
	}

	/**
	 * @return string representation
	 */
	public String toString() {
		return "Skinning [joints="
				+ joints + ", vertextJointsWeights=" + verticesJointsWeights
				+ "]";
	}

}