package net.drewke.tdme.physics;

import net.drewke.tdme.math.Vector3D;

/**
 * Collision response
 * @author Andreas Drewke
 * @version $Id: aea9fca7d24fd965827622b15137bb673a1ca80f $
 */
public class CollisionResponse {

	private float distance;
	private Vector3D normal;

	/**
	 * Public constructor
	 * @param distance
	 * @param normal
	 */
	public CollisionResponse(float distance, Vector3D normal) {
		this.distance = distance;
		this.normal = normal;
	}

	/**
	 * @return distance
	 */
	public float getDistance() {
		return distance;
	}

	/**
	 * @return if we have a penetration
	 */
	public boolean hasPenetration() {
		return distance < 0f;
	}

	/**
	 * @return penetration depth
	 */
	public float getPenetration() {
		return -distance;
	}

	/**
	 * @return normal of collision
	 */
	public Vector3D getNormal() {
		return normal;
	}

	/*
	 * (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString() {
		return "CollisionResponse [distance=" + distance + ", normal=" + normal
				+ ", hasCollision()=" + hasPenetration() + ", getPenetration()="
				+ getPenetration() + "]";
	}

}
