package net.drewke.tdme.primitives;

import net.drewke.tdme.engine.Transformations;
import net.drewke.tdme.math.Vector3D;
import net.drewke.tdme.physics.CollisionResponse;

/**
 * Bounding Volume Interface
 * @author Andreas Drewke
 * @version $Id: 8e59e3d6bafdcaa685cc2c581bb3a9623c2bc64a $
 */
public interface BoundingVolume {

	/**
	 * Create bounding volume from given original(of same type) with applied transformations
	 * @param original bounding volume
	 * @param transformations
	 */
	public void fromBoundingVolumeWithTransformations(BoundingVolume original, Transformations transformations);

	/**
	 * Computes closest point on bounding volume for given point 
	 * @param point
	 * @return closed point of this bounding volume
	 */
	public Vector3D computeClosestPointOnBoundingVolume(Vector3D point);

	/**
	 * Checks if point is in bounding volume
	 * @param point
	 * @return bool if point is in bounding volume
	 */
	public boolean containsPoint(Vector3D point);

	/**
	 * Check if this bounding volume collides with bounding volume 2
	 * @param bounding volume 2
	 * @return collision response or null
	 */
	public CollisionResponse doesCollideWith(BoundingVolume bv2);

	/**
	 * Clones this bounding volume
	 * @return cloned bounding volume
	 */
	public BoundingVolume clone();

}
