package net.drewke.tdme.test;

import java.awt.Frame;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.util.Vector;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.media.opengl.GLAutoDrawable;
import javax.media.opengl.GLCapabilities;
import javax.media.opengl.GLEventListener;
import javax.media.opengl.GLProfile;
import javax.media.opengl.awt.GLCanvas;

import net.drewke.tdme.engine.Camera;
import net.drewke.tdme.engine.Engine;
import net.drewke.tdme.engine.Light;
import net.drewke.tdme.engine.ModelUtilities;
import net.drewke.tdme.engine.Object3D;
import net.drewke.tdme.engine.Rotation;
import net.drewke.tdme.math.Vector3D;
import net.drewke.tdme.model.Face;
import net.drewke.tdme.model.FacesEntity;
import net.drewke.tdme.model.Group;
import net.drewke.tdme.model.Material;
import net.drewke.tdme.model.Model;
import net.drewke.tdme.model.TextureCoordinate;
import net.drewke.tdme.parser.DAEParser;
import net.drewke.tdme.physics.CollisionResponse;
import net.drewke.tdme.primitives.BoundingVolume;
import net.drewke.tdme.primitives.OrientedBoundingBox;

import com.jogamp.opengl.util.FPSAnimator;

/**
 * Engine test
 * @author andreas.drewke
 * @version $Id: c7b4403037edca36a80407d5ab07b46258839954 $
 */
public class EngineTest implements GLEventListener, MouseListener, MouseMotionListener, KeyListener {

	private Engine engine;

	private Vector<Object3D> players;
	private BoundingVolume playerBoundingVolume;
	private Vector<BoundingVolume> playerBoundingVolumesTransformed;

	private Object3D cube;
	private BoundingVolume cubeBoundingVolume;
	private BoundingVolume cubeBoundingVolumeTransformed;

	private boolean showBoundingVolumes = true;

	private boolean keyLeft;
	private boolean keyRight;
	private boolean keyUp;

	private boolean keyW;
	private boolean keyA;
	private boolean keyS;
	private boolean keyD;

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		Logger.getLogger("").setLevel(Level.WARNING);

		// create GL canvas
		GLProfile glp = Engine.getProfile();
		GLCapabilities caps = new GLCapabilities(glp);
		caps.setDepthBits(16);
		final GLCanvas glCanvas = new GLCanvas(caps);

		// create AWT frame
		final Frame frame = new Frame("Engine Test");
		frame.setSize(640, 480);
		frame.add(glCanvas);
		frame.setVisible(true);

		// add window listener to be able to handle window close events
		frame.addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				frame.remove(glCanvas);
				frame.dispose();
				System.exit(0);
			}
		});

		// event listener
		EngineTest renderTest = new EngineTest(glCanvas);
		glCanvas.addGLEventListener(renderTest);
		glCanvas.addMouseListener(renderTest);
		glCanvas.addMouseMotionListener(renderTest);
		glCanvas.addKeyListener(renderTest);
		glCanvas.setEnabled(true);

		// animator, do the frames
		FPSAnimator animator = new FPSAnimator(glCanvas, 60);
		animator.start();
	}

	/**
	 * Public constructor
	 * 
	 * @param gl
	 *            canvas
	 */
	public EngineTest(GLCanvas glCanvas) {
		keyLeft = false;
		keyRight = false;
		keyUp = false;
		keyW = false;
		keyA = false;
		keyS = false;
		keyD = false;
		engine = new Engine();
	}

	private Model createGroundModel() {
		// ground model
		Model ground = new Model("ground", "ground");

		//	material
		Material groundMaterial = new Material("ground");
		groundMaterial.setupDiffuseTexture("resources/environment/test2terrain", "Boden_Wald_1b.jpg");
		ground.getMaterials().put("ground", groundMaterial);

		//	group
		Group groundGroup = new Group(ground, "ground", "ground");

		//	faces entity
		//		ground
		FacesEntity groupFacesEntityGround = new FacesEntity(groundGroup, "ground group faces entity ground");
		groupFacesEntityGround.setMaterial(groundMaterial);

		//		far plane
		FacesEntity groupFacesEntityFarPlane = new FacesEntity(groundGroup, "ground group faces entity far plane");
		groupFacesEntityFarPlane.setMaterial(Material.getDefaultMaterial());

		//	faces entity 
		Vector<FacesEntity> groupFacesEntities = new Vector<FacesEntity>();
		groupFacesEntities.add(groupFacesEntityGround);
		groupFacesEntities.add(groupFacesEntityFarPlane);

		//	vertices
		Vector<Vector3D> groundVertices = new Vector<Vector3D>();
		// left, near, ground
		groundVertices.add(new Vector3D(-400f, 0f, -400f));
		// left, far, ground
		groundVertices.add(new Vector3D(-400f, 0f, +400f));
		// right far, ground
		groundVertices.add(new Vector3D(+400f, 0f, +400f));
		// right, near, ground
		groundVertices.add(new Vector3D(+400f, 0f, -400f));

		// left, near, far plane
		groundVertices.add(new Vector3D(-400f, 0f, 400f));
		// left, far, far plane
		groundVertices.add(new Vector3D(-400f, 200f, 400f));
		// right far, far plane
		groundVertices.add(new Vector3D(+400f, 200f, 400f));
		// right, near, far plane
		groundVertices.add(new Vector3D(+400f, 0f, 400f));

		//	normals
		Vector<Vector3D> groundNormals = new Vector<Vector3D>();
		//		ground
		groundNormals.add(new Vector3D(0f, 1f, 0f));
		//		far plane
		groundNormals.add(new Vector3D(0f, 0f, 1f));

		// texture coordinates
		Vector<TextureCoordinate> groundTextureCoordinates = new Vector<TextureCoordinate>();
		groundTextureCoordinates.add(new TextureCoordinate(0f, 0f));
		groundTextureCoordinates.add(new TextureCoordinate(0f, 1f));
		groundTextureCoordinates.add(new TextureCoordinate(1f, 1f));
		groundTextureCoordinates.add(new TextureCoordinate(1f, 0f));

		//	faces ground
		Vector<Face> groundFacesGround = new Vector<Face>();
		groundFacesGround.add(new Face(groundGroup,0,1,2,0,0,0,0,1,2));
		groundFacesGround.add(new Face(groundGroup,2,3,0,0,0,0,2,3,0));

		// set up faces entity
		groupFacesEntityGround.setupFaces(groundFacesGround);

		//	faces ground far plane
		Vector<Face> groundFacesFarPlane = new Vector<Face>();
		groundFacesFarPlane.add(new Face(groundGroup,4,5,6,1,1,1,0,1,2));
		groundFacesFarPlane.add(new Face(groundGroup,6,7,4,1,1,1,2,3,0));

		// set up faces entity
		groupFacesEntityFarPlane.setupFaces(groundFacesFarPlane);

		// setup ground group
		groundGroup.setupVertices(groundVertices);
		groundGroup.setupNormals(groundNormals);
		groundGroup.setupTextureCoordinates(groundTextureCoordinates);
		groundGroup.setupFacesEntities(groupFacesEntities);

		// register group
		ground.getGroups().put("ground", groundGroup);
		ground.getSubGroups().put("ground", groundGroup);

		//
		System.out.println(ground);

		//
		return ground;
	}

	public void display(GLAutoDrawable drawable) {
		// player control
		doPlayerControl(0, keyLeft, keyRight, keyUp);
		doPlayerControl(1, keyA, keyD, keyW);

		engine.display(drawable);

		/*
		if (showBoundingVolumes == true) {
			GL2 gl = drawable.getGL().getGL2();
			Primitives.doRenderBoundingVolume(gl, cubeBoundingVolumeTransformed);
			Primitives.doRenderBoundingVolume(gl, playerBoundingVolumesTransformed.get(0));
			Primitives.doRenderBoundingVolume(gl, playerBoundingVolumesTransformed.get(1));
		}
		*/
	}

	private void doPlayerControl(int idx, boolean keyLeft, boolean keyRight, boolean keyUp) {
		float fps = engine.getTiming().getCurrentFPS();

		Object3D player = players.get(idx);
		BoundingVolume playerBoundingVolumeTransformed = playerBoundingVolumesTransformed.get(idx);

		Vector<Rotation> rotations = player.getRotations();
		Rotation r = rotations.elementAt(0);

		// left, right
		if (keyRight) r.setAngle(r.getAngle() + (135f / fps));
		if (keyLeft) r.setAngle(r.getAngle() - (135f / fps));
		if (keyRight || keyLeft) {
			player.update();
			playerBoundingVolumeTransformed.fromBoundingVolumeWithTransformations(playerBoundingVolume, player);
		}

		// forward
		if (keyUp) {
			// apply movement
			Vector3D movement = r.getMatrix().multiply(new Vector3D(0f,0f,1f)).scale((150f / fps));
			player.getTranslation().add(
				movement
			);
			player.update();
			playerBoundingVolumeTransformed.fromBoundingVolumeWithTransformations(playerBoundingVolume, player);

			// walking animation
			if (player.getAnimation().equals("walk") == false) {
				player.setAnimation("walk");
			}
		} else {
			// still animation
			if (player.getAnimation().equals("walk") == true) {
				player.setAnimation("still");
			}			
		}

		//
		CollisionResponse collision;

		// check if collides with cube
		collision = playerBoundingVolumeTransformed.doesCollideWith(cubeBoundingVolumeTransformed);
		if (collision != null && collision.hasPenetration()) {
			System.out.println("cube: " + collision);
			// yep, move object out of collision 
			player.getTranslation().sub(
				collision.getNormal().clone().scale(collision.getPenetration())
			);
			player.update();
			playerBoundingVolumeTransformed.fromBoundingVolumeWithTransformations(playerBoundingVolume, player);				
		}

		// check if collides with other players
		for (int i = 0; i < players.size(); i++) {
			// do not check with same player
			if (idx == i) continue;

			// do collide?
			collision = playerBoundingVolumeTransformed.doesCollideWith(playerBoundingVolumesTransformed.get(i));
			if (collision != null && collision.hasPenetration()) {
				System.out.println("player: " + collision);

				// yep, move object out of collision
				player.getTranslation().sub(
					collision.getNormal().clone().scale(collision.getPenetration())
				);
				player.update();
				playerBoundingVolumeTransformed.fromBoundingVolumeWithTransformations(playerBoundingVolume, player);
			}
		}
	}

	public void dispose(GLAutoDrawable drawable) {
		engine.dispose(drawable);
	}

	public void init(GLAutoDrawable drawable) {
		engine.init(drawable);

		// cam
		Camera cam = engine.getCamera();
		cam.setZNear(10f);
		cam.setZFar(5000f);
		cam.getLookFrom().set(0f, 800f, -400f);
		cam.getLookAt().set(0f, 50f, 0f);
		cam.getUpVector().set(Camera.computeUpVector(cam.getLookFrom(), cam.getLookAt()));

		// lights
		Light light0 = engine.getLightAt(0);
		light0.setAmbient(0.8f, 0.8f, 0.8f, 1f);
		light0.setEnabled(true);
		Light light1 = engine.getLightAt(1);
		light1.setDiffuse(0.4f, 0.0f, 0.0f, 1.0f);
		light1.getPosition().set(-500f,500f, -500f, 1.0f);
		light1.setEnabled(true);
		Light light2 = engine.getLightAt(2);
		light2.setDiffuse(0.0f, 0.4f, 0.0f, 0.0f);
		light2.getPosition().set(+1000f,1000f, 0f, 0f);
		light2.setEnabled(true);

		// scene
		players = new Vector<Object3D>();
		playerBoundingVolumesTransformed = new Vector<BoundingVolume>();
		try {
			// ground
			Model _ground = createGroundModel();
			engine.addObject(new Object3D("ground", _ground));

			// players
			Model _player = DAEParser.parse("resources/models/dummy", "testDummy_textured.DAE");
			_player.addAnimationSetup("still", 3, 3, true);
			_player.addAnimationSetup("walk", 0, 18, true);

			// player capsule
			// playerBoundingVolume = new Capsule(new Vector3D(0,15,0), new Vector3D(0,160,0), 25);
			// playerBoundingVolume = new BoundingBox(new Vector3D(-25f, 0, -25f), new Vector3D(+25f, 160f, +25f));
			playerBoundingVolume = new OrientedBoundingBox(
				new Vector3D(0f, 80f, 0f),
				new Vector3D(1f, 0f, 0f),
				new Vector3D(0f, 1f, 0f),
				new Vector3D(0f, 0f, 1f),
				new Vector3D(25f, 80f, 25f)
			);

			// add player 1
			//	player
			Object3D player1 = new Object3D("player1", _player);
			player1.getTranslation().add(new Vector3D(-150f,0f,0f));
			player1.setAnimation("still");
			player1.getRotations().add(new Rotation(0f, new Vector3D(0f, 1f, 0f)));
			player1.update();
			player1.setDynamicShadowingEnabled(true);
			player1.setShowVisibilityVolume(false);
			engine.addObject(player1);
			//	bounding volume transformed
			BoundingVolume player1BoundingVolumeTransformed = playerBoundingVolume.clone();
			player1BoundingVolumeTransformed.fromBoundingVolumeWithTransformations(playerBoundingVolume, player1);
			playerBoundingVolumesTransformed.add(player1BoundingVolumeTransformed);
			//	add to engine
			players.add(player1);

			// add player 2
			//	player
			Object3D player2 = new Object3D("player2", _player);
			player2.getTranslation().add(new Vector3D(150f,0f,0f));
			player2.setAnimation("still");
			player2.getRotations().add(new Rotation(0f, new Vector3D(0f, 1f, 0f)));
			player2.update();
			player2.setShowVisibilityVolume(false);
			player2.setDynamicShadowingEnabled(true);
			players.add(player2);
			//	bounding volume transformed
			BoundingVolume player2BoundingVolumeTransformed = playerBoundingVolume.clone();
			player2BoundingVolumeTransformed.fromBoundingVolumeWithTransformations(playerBoundingVolume, player2);
			playerBoundingVolumesTransformed.add(player2BoundingVolumeTransformed);
			//	add to engine
			engine.addObject(player2);

			// add cube
			Model _cube = DAEParser.parse("resources/models/test", "cube.dae");
			cube = new Object3D("cube", _cube);
			cube.getTranslation().add(new Vector3D(0f,0f,0f));
			cube.update();
			cube.setShowVisibilityVolume(false);
			cube.setDynamicShadowingEnabled(true);
			cubeBoundingVolume = cube.getBoundingBox();
			cubeBoundingVolumeTransformed = cubeBoundingVolume.clone();
			cubeBoundingVolumeTransformed.fromBoundingVolumeWithTransformations(cubeBoundingVolume, cube);
			engine.addObject(cube);

			Model _wall = DAEParser.parse("resources/models/wall", "wall.dae");
			Object3D wall0 = new Object3D("wall0", _wall);
			wall0.getTranslation().add(new Vector3D(-100f,0f,300f));
			wall0.getScale().set(100f,100f,100f);
			wall0.update();
			engine.addObject(wall0);
			Object3D wall1 = new Object3D("wall1", _wall);
			wall1.getTranslation().add(new Vector3D(0f,0f,300f));
			wall1.getScale().set(100f,100f,100f);
			wall1.update();
			engine.addObject(wall1);
		} catch (Exception exception) {
			exception.printStackTrace();
			System.out.println("Could not load object: " + exception.getMessage());
		}
	}

	public void reshape(GLAutoDrawable drawable, int x, int y, int width, int height) {
		engine.reshape(drawable, x, y, width, height);
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent e) {
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent e) {
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent e) {
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent e) {
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent e) {
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseDragged(java.awt.event.MouseEvent)
	 */
	public void mouseDragged(MouseEvent e) {
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.MouseMotionListener#mouseMoved(java.awt.event.MouseEvent)
	 */
	public void mouseMoved(MouseEvent e) {
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
	 */
	public void keyPressed(KeyEvent e) {
		int keyCode = e.getKeyCode();
		char keyChar = Character.toLowerCase(e.getKeyChar());
		if (keyChar == 'w') keyW = true;
		if (keyChar == 'a') keyA = true;
		if (keyChar == 's') keyS = true;
		if (keyChar == 'd') keyD = true;
		if (keyCode == KeyEvent.VK_LEFT) keyLeft = true;
		if (keyCode == KeyEvent.VK_RIGHT) keyRight = true;
		if (keyCode == KeyEvent.VK_UP) keyUp = true;
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 */
	public void keyReleased(KeyEvent e) {
		int keyCode = e.getKeyCode();
		char keyChar = Character.toLowerCase(e.getKeyChar());
		if (keyChar == 'w') keyW = false;
		if (keyChar == 'a') keyA = false;
		if (keyChar == 's') keyS = false;
		if (keyChar == 'd') keyD = false;
		if (keyCode == KeyEvent.VK_LEFT) keyLeft = false;
		if (keyCode == KeyEvent.VK_RIGHT) keyRight = false;
		if (keyCode == KeyEvent.VK_UP) keyUp = false;
	}

	/*
	 * (non-Javadoc)
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 */
	public void keyTyped(KeyEvent e) {
		char keyChar = Character.toLowerCase(e.getKeyChar());
		if (keyChar == 'b') showBoundingVolumes = !showBoundingVolumes;
	}

}